@@ -5,15 +5,22 @@ from dagster.config.config_type import Array, Noneable, ScalarUnion
 from dagster.core.definitions import (
     AssetKey,
     AssetMaterialization,
+    AssetSensorDefinition,
     CompositeSolidDefinition,
     ConfigMapping,
     DependencyDefinition,
+    DynamicOut,
+    DynamicOutput,
+    DynamicOutputDefinition,
+    EventMetadata,
     EventMetadataEntry,
     ExecutorDefinition,
+    ExecutorRequirement,
     ExpectationResult,
     Failure,
     FloatMetadataEntryData,
     HookDefinition,
+    In,
     InputDefinition,
     InputMapping,
     IntMetadataEntryData,
@@ -24,21 +31,27 @@ from dagster.core.definitions import (
     Materialization,
     ModeDefinition,
     MultiDependencyDefinition,
+    Out,
     Output,
     OutputDefinition,
     OutputMapping,
     Partition,
+    PartitionScheduleDefinition,
     PartitionSetDefinition,
     PathMetadataEntryData,
     PipelineDefinition,
+    PipelineFailureSensorContext,
     PresetDefinition,
+    PythonArtifactMetadataEntryData,
     RepositoryDefinition,
     ResourceDefinition,
     RetryRequested,
     RunRequest,
     ScheduleDefinition,
+    ScheduleEvaluationContext,
     ScheduleExecutionContext,
     SensorDefinition,
+    SensorEvaluationContext,
     SensorExecutionContext,
     SkipReason,
     SolidDefinition,
@@ -46,19 +59,26 @@ from dagster.core.definitions import (
     TextMetadataEntryData,
     TypeCheck,
     UrlMetadataEntryData,
+    asset_sensor,
+    build_init_logger_context,
     composite_solid,
     daily_schedule,
     default_executors,
     executor,
     failure_hook,
+    graph,
     hourly_schedule,
     in_process_executor,
     intermediate_storage,
     lambda_solid,
     logger,
+    make_values_resource,
     monthly_schedule,
+    multiple_process_executor_requirements,
     multiprocess_executor,
+    op,
     pipeline,
+    pipeline_failure_sensor,
     reconstructable,
     repository,
     resource,
@@ -69,6 +89,9 @@ from dagster.core.definitions import (
     weekly_schedule,
 )
 from dagster.core.definitions.configurable import configured
+from dagster.core.definitions.policy import Backoff, Jitter, RetryPolicy
+from dagster.core.definitions.schedule import build_schedule_context
+from dagster.core.definitions.sensor import build_sensor_context
 from dagster.core.errors import (
     DagsterConfigMappingFunctionError,
     DagsterError,
@@ -97,31 +120,36 @@ from dagster.core.execution.api import (
     reexecute_pipeline_iterator,
 )
 from dagster.core.execution.context.compute import SolidExecutionContext
-from dagster.core.execution.context.init import InitResourceContext
+from dagster.core.execution.context.hook import HookContext, build_hook_context
+from dagster.core.execution.context.init import InitResourceContext, build_init_resource_context
+from dagster.core.execution.context.input import InputContext, build_input_context
+from dagster.core.execution.context.invocation import build_op_context, build_solid_context
 from dagster.core.execution.context.logger import InitLoggerContext
-from dagster.core.execution.context.system import (
-    HookContext,
-    InputContext,
-    OutputContext,
-    SystemComputeExecutionContext,
-    TypeCheckContext,
-)
+from dagster.core.execution.context.output import OutputContext, build_output_context
+from dagster.core.execution.context.system import TypeCheckContext
 from dagster.core.execution.results import (
     CompositeSolidExecutionResult,
     PipelineExecutionResult,
     SolidExecutionResult,
 )
+from dagster.core.execution.validate_run_config import validate_run_config
 from dagster.core.executor.base import Executor
 from dagster.core.executor.init import InitExecutorContext
 from dagster.core.instance import DagsterInstance
 from dagster.core.launcher import DefaultRunLauncher
 from dagster.core.log_manager import DagsterLogManager
+from dagster.core.storage.event_log import (
+    EventLogEntry,
+    EventLogRecord,
+    EventRecordsFilter,
+    RunShardedEventsCursor,
+)
 from dagster.core.storage.file_manager import FileHandle, LocalFileHandle, local_file_manager
 from dagster.core.storage.fs_io_manager import custom_path_fs_io_manager, fs_io_manager
 from dagster.core.storage.init import InitIntermediateStorageContext
 from dagster.core.storage.io_manager import IOManager, IOManagerDefinition, io_manager
 from dagster.core.storage.mem_io_manager import mem_io_manager
-from dagster.core.storage.pipeline_run import PipelineRun
+from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
 from dagster.core.storage.root_input_manager import (
     RootInputManager,
     RootInputManagerDefinition,
@@ -172,44 +200,63 @@ __all__ = [
     # Definition
     "AssetKey",
     "AssetMaterialization",
+    "AssetSensorDefinition",
     "CompositeSolidDefinition",
     "ConfigMapping",
     "DependencyDefinition",
+    "EventMetadata",
     "EventMetadataEntry",
     "ExecutorDefinition",
+    "ExecutorRequirement",
     "ExpectationResult",
     "Failure",
     "Field",
     "HookDefinition",
+    "In",
     "InputDefinition",
     "InputMapping",
     "IntermediateStorageDefinition",
     "JsonMetadataEntryData",
     "LoggerDefinition",
+    "build_init_logger_context",
     "MarkdownMetadataEntryData",
     "IntMetadataEntryData",
     "FloatMetadataEntryData",
     "Materialization",
     "ModeDefinition",
     "MultiDependencyDefinition",
+    "Out",
     "Output",
     "OutputDefinition",
     "OutputMapping",
     "PathMetadataEntryData",
     "PipelineDefinition",
     "PresetDefinition",
+    "PythonArtifactMetadataEntryData",
     "RepositoryDefinition",
     "ResourceDefinition",
     "SolidDefinition",
     "SolidInvocation",
     "TextMetadataEntryData",
     "UrlMetadataEntryData",
+    "make_values_resource",
+    "RetryPolicy",
+    "Backoff",
+    "Jitter",
+    "PipelineFailureSensorContext",
+    "pipeline_failure_sensor",
+    "DynamicOutput",
+    "DynamicOut",
+    "DynamicOutputDefinition",
     # Decorators
+    "asset_sensor",
     "composite_solid",
     "executor",
+    "graph",
     "intermediate_storage",
     "lambda_solid",
     "logger",
+    "op",
     "pipeline",
     "repository",
     "resource",
@@ -223,31 +270,41 @@ __all__ = [
     "DagsterEvent",
     "DagsterEventType",
     "DefaultRunLauncher",
+    "EventLogEntry",
+    "EventLogRecord",
     "Executor",
     "InitExecutorContext",
     "InitLoggerContext",
     "InitResourceContext",
+    "build_init_resource_context",
     "InitIntermediateStorageContext",
     "PipelineExecutionResult",
     "RetryRequested",
     "SolidExecutionResult",
-    "SystemComputeExecutionContext",
     "SolidExecutionContext",
+    "build_solid_context",
+    "build_op_context",
     "HookContext",
+    "build_hook_context",
     "TypeCheckContext",
     "InputContext",
+    "build_input_context",
     "OutputContext",
+    "build_output_context",
     "PipelineRun",
+    "PipelineRunStatus",
     "default_executors",
     "default_intermediate_storage_defs",
     "execute_pipeline_iterator",
     "execute_pipeline",
+    "validate_run_config",
     "execute_solid_within_pipeline",
     "fs_intermediate_storage",
     "in_process_executor",
     "mem_intermediate_storage",
     "io_manager_from_intermediate_storage",
     "multiprocess_executor",
+    "multiple_process_executor_requirements",
     "reconstructable",
     "reexecute_pipeline_iterator",
     "reexecute_pipeline",
@@ -321,14 +378,22 @@ __all__ = [
     "local_file_manager",
     # instance
     "DagsterInstance",
+    # storage
+    "EventRecordsFilter",
+    "RunShardedEventsCursor",
     # partitions and schedules
     "Partition",
+    "PartitionScheduleDefinition",
     "PartitionSetDefinition",
     "RunRequest",
     "ScheduleDefinition",
+    "ScheduleEvaluationContext",
     "ScheduleExecutionContext",
+    "build_schedule_context",
     "SensorDefinition",
+    "SensorEvaluationContext",
     "SensorExecutionContext",
+    "build_sensor_context",
     "SkipReason",
     "daily_schedule",
     "hourly_schedule",
