@@ -2,7 +2,6 @@ from collections import namedtuple
 from enum import Enum
 
 from dagster import check
-from dagster.core.errors import DagsterBackfillFailedError
 from dagster.core.execution.plan.resume_retry import get_retry_steps_from_execution_plan
 from dagster.core.execution.plan.state import KnownExecutionState
 from dagster.core.host_representation import (
@@ -11,15 +10,21 @@ from dagster.core.host_representation import (
     RepositoryLocation,
 )
 from dagster.core.host_representation.external_data import (
-    ExternalPartitionExecutionErrorData,
     ExternalPartitionExecutionParamData,
     ExternalPartitionSetExecutionParamData,
 )
 from dagster.core.host_representation.origin import ExternalPartitionSetOrigin
 from dagster.core.instance import DagsterInstance
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
-from dagster.core.storage.tags import PARTITION_NAME_TAG, PARTITION_SET_TAG, RESUME_RETRY_TAG
+from dagster.core.storage.tags import (
+    PARENT_RUN_ID_TAG,
+    PARTITION_NAME_TAG,
+    PARTITION_SET_TAG,
+    RESUME_RETRY_TAG,
+    ROOT_RUN_ID_TAG,
+)
 from dagster.core.utils import make_new_run_id
+from dagster.core.workspace.workspace import IWorkspace
 from dagster.serdes import whitelist_for_serdes
 from dagster.utils import merge_dicts
 from dagster.utils.error import SerializableErrorInfo
@@ -118,8 +123,9 @@ class PartitionBackfill(
         )
 
 
-def submit_backfill_runs(instance, repo_location, backfill_job, partition_names=None):
+def submit_backfill_runs(instance, workspace, repo_location, backfill_job, partition_names=None):
     check.inst_param(instance, "instance", DagsterInstance)
+    check.inst_param(workspace, "workspace", IWorkspace)
     check.inst_param(repo_location, "repo_location", RepositoryLocation)
     check.inst_param(backfill_job, "backfill_job", PartitionBackfill)
 
@@ -141,8 +147,6 @@ def submit_backfill_runs(instance, repo_location, backfill_job, partition_names=
     result = repo_location.get_external_partition_set_execution_param_data(
         external_repo.handle, partition_set_name, partition_names
     )
-    if isinstance(result, ExternalPartitionExecutionErrorData):
-        raise DagsterBackfillFailedError(serializable_error_info=result.error)
 
     assert isinstance(result, ExternalPartitionSetExecutionParamData)
     external_pipeline = external_repo.get_full_external_pipeline(
@@ -161,7 +165,7 @@ def submit_backfill_runs(instance, repo_location, backfill_job, partition_names=
             # we skip runs in certain cases, e.g. we are running a `from_failure` backfill job
             # and the partition has had a successful run since the time the backfill was
             # scheduled
-            instance.submit_run(pipeline_run.run_id, external_pipeline)
+            instance.submit_run(pipeline_run.run_id, workspace)
             yield pipeline_run.run_id
         yield None
 
@@ -176,7 +180,6 @@ def create_backfill_run(
     check.inst_param(backfill_job, "backfill_job", PartitionBackfill)
     check.inst_param(partition_data, "partition_data", ExternalPartitionExecutionParamData)
 
-    # potentially wrap this so we don't waste resources if plan is bad
     full_external_execution_plan = repo_location.get_external_execution_plan(
         external_pipeline,
         partition_data.run_config,
@@ -205,7 +208,14 @@ def create_backfill_run(
 
         parent_run_id = last_run.run_id
         root_run_id = last_run.root_run_id or last_run.run_id
-        tags = merge_dicts(tags, {RESUME_RETRY_TAG: "true"})
+        tags = merge_dicts(
+            tags,
+            {
+                RESUME_RETRY_TAG: "true",
+                PARENT_RUN_ID_TAG: parent_run_id,
+                ROOT_RUN_ID_TAG: root_run_id,
+            },
+        )
         step_keys_to_execute, known_state = get_retry_steps_from_execution_plan(
             instance, full_external_execution_plan, parent_run_id
         )
@@ -213,6 +223,10 @@ def create_backfill_run(
         last_run = _fetch_last_run(instance, external_partition_set, partition_data.name)
         parent_run_id = last_run.run_id if last_run else None
         root_run_id = (last_run.root_run_id or last_run.run_id) if last_run else None
+        if parent_run_id and root_run_id:
+            tags = merge_dicts(
+                tags, {PARENT_RUN_ID_TAG: parent_run_id, ROOT_RUN_ID_TAG: root_run_id}
+            )
         step_keys_to_execute = backfill_job.reexecution_steps
         if last_run and last_run.status == PipelineRunStatus.SUCCESS:
             known_state = KnownExecutionState.for_reexecution(
@@ -250,6 +264,7 @@ def create_backfill_run(
         parent_run_id=parent_run_id,
         status=PipelineRunStatus.NOT_STARTED,
         external_pipeline_origin=external_pipeline.get_external_origin(),
+        pipeline_code_origin=external_pipeline.get_python_origin(),
     )
 
 
