@@ -7,29 +7,35 @@ import time
 import warnings
 import weakref
 from collections import defaultdict
-from datetime import datetime
 from enum import Enum
-from typing import TYPE_CHECKING, Dict, Iterable, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Union, cast
 
 import yaml
 from dagster import check
 from dagster.core.definitions.events import AssetKey
 from dagster.core.definitions.pipeline import PipelineDefinition, PipelineSubsetDefinition
+from dagster.core.definitions.pipeline_base import InMemoryPipeline
 from dagster.core.errors import (
+    DagsterHomeNotSetError,
     DagsterInvariantViolationError,
     DagsterNoStepsToExecuteException,
     DagsterRunAlreadyExists,
     DagsterRunConflict,
 )
-from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
+from dagster.core.storage.pipeline_run import (
+    PipelineRun,
+    PipelineRunStatus,
+    PipelineRunsFilter,
+    RunRecord,
+)
 from dagster.core.storage.tags import MEMOIZED_RUN_TAG
-from dagster.core.system_config.objects import EnvironmentConfig
+from dagster.core.system_config.objects import ResolvedRunConfig
 from dagster.core.utils import str_format_list
 from dagster.serdes import ConfigurableClass
 from dagster.seven import get_current_datetime_in_utc
 from dagster.utils.error import serializable_error_info_from_exc_info
 
-from .config import DAGSTER_CONFIG_YAML_FILENAME
+from .config import DAGSTER_CONFIG_YAML_FILENAME, is_dagster_home_set
 from .ref import InstanceRef
 
 # 'airflow_execution_date' and 'is_airflow_ingest_pipeline' are hardcoded tags used in the
@@ -42,59 +48,18 @@ IS_AIRFLOW_INGEST_PIPELINE_STR = "is_airflow_ingest_pipeline"
 
 
 if TYPE_CHECKING:
-    from dagster.core.events import DagsterEvent
+    from dagster.core.events import DagsterEvent, DagsterEventType
     from dagster.core.host_representation import HistoricalPipeline
     from dagster.core.snap import PipelineSnapshot
+    from dagster.core.storage.event_log.base import EventRecordsFilter, EventLogRecord
+    from dagster.core.workspace.workspace import IWorkspace
     from dagster.daemon.types import DaemonHeartbeat
 
 
-def _is_dagster_home_set():
-    return bool(os.getenv("DAGSTER_HOME"))
-
-
 def is_memoized_run(tags):
     return tags is not None and MEMOIZED_RUN_TAG in tags and tags.get(MEMOIZED_RUN_TAG) == "true"
 
 
-def _dagster_home():
-    dagster_home_path = os.getenv("DAGSTER_HOME")
-
-    if not dagster_home_path:
-        raise DagsterInvariantViolationError(
-            (
-                "The environment variable $DAGSTER_HOME is not set. Dagster requires this "
-                "environment variable to be set to an existing directory in your filesystem "
-                "that contains your dagster instance configuration file (dagster.yaml).\n"
-                "You can resolve this error by exporting the environment variable."
-                "For example, you can run the following command in your shell or "
-                "include it in your shell configuration file:\n"
-                '\texport DAGSTER_HOME="~/dagster_home"'
-            )
-        )
-
-    dagster_home_path = os.path.expanduser(dagster_home_path)
-
-    if not os.path.isabs(dagster_home_path):
-        raise DagsterInvariantViolationError(
-            (
-                '$DAGSTER_HOME "{}" must be an absolute path. Dagster requires this '
-                "environment variable to be set to an existing directory in your filesystem that "
-                "contains your dagster instance configuration file (dagster.yaml)."
-            ).format(dagster_home_path)
-        )
-
-    if not (os.path.exists(dagster_home_path) and os.path.isdir(dagster_home_path)):
-        raise DagsterInvariantViolationError(
-            (
-                '$DAGSTER_HOME "{}" is not a directory or does not exist. Dagster requires this '
-                "environment variable to be set to an existing directory in your filesystem that "
-                "contains your dagster instance configuration file (dagster.yaml)."
-            ).format(dagster_home_path)
-        )
-
-    return dagster_home_path
-
-
 def _check_run_equality(pipeline_run, candidate_run):
     check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
     check.inst_param(candidate_run, "candidate_run", PipelineRun)
@@ -165,19 +130,20 @@ class MayHaveInstanceWeakref:
     """Mixin for classes that can have a weakref back to a Dagster instance."""
 
     def __init__(self):
-        self._instance_weakref: weakref.ref = None
+        self._instance_weakref: weakref.ReferenceType["DagsterInstance"] = None
 
     @property
-    def _instance(self):
-        return (
+    def _instance(self) -> "DagsterInstance":
+        instance = (
             self._instance_weakref()
             # Backcompat with custom subclasses that don't call super().__init__()
             # in their own __init__ implementations
             if (hasattr(self, "_instance_weakref") and self._instance_weakref is not None)
             else None
         )
+        return cast("DagsterInstance", instance)
 
-    def register_instance(self, instance):
+    def register_instance(self, instance: "DagsterInstance"):
         check.inst_param(instance, "instance", DagsterInstance)
         check.invariant(
             # Backcompat with custom subclasses that don't call super().__init__()
@@ -194,8 +160,10 @@ class DagsterInstance:
     """Core abstraction for managing Dagster's access to storage and other resources.
 
     Use DagsterInstance.get() to grab the current DagsterInstance which will load based on
-    the values in the ``dagster.yaml`` file in ``$DAGSTER_HOME`` if set, otherwise fallback
-    to using an ephemeral in-memory set of components.
+    the values in the ``dagster.yaml`` file in ``$DAGSTER_HOME``.
+
+    Alternatively, DagsterInstance.ephemeral() can use used which provides a set of
+    transient in-memory components.
 
     Configuration of this class should be done by setting values in ``$DAGSTER_HOME/dagster.yaml``.
     For example, to use Postgres for run and event log storage, you can write a ``dagster.yaml``
@@ -277,6 +245,7 @@ class DagsterInstance:
         self._compute_log_manager = check.inst_param(
             compute_log_manager, "compute_log_manager", ComputeLogManager
         )
+        self._compute_log_manager.register_instance(self)
         self._scheduler = check.opt_inst_param(scheduler, "scheduler", Scheduler)
 
         self._schedule_storage = check.opt_inst_param(
@@ -346,28 +315,44 @@ class DagsterInstance:
         )
 
     @staticmethod
-    def get(fallback_storage=None):
-        # 1. Use $DAGSTER_HOME to determine instance if set.
-        if _is_dagster_home_set():
-            return DagsterInstance.from_config(_dagster_home())
+    def get():
+        dagster_home_path = os.getenv("DAGSTER_HOME")
 
-        # 2. If that is not set use the fallback storage directory if provided.
-        # This allows us to have a nice out of the box dagit experience where runs are persisted
-        # across restarts in a tempdir that gets cleaned up when the dagit watchdog process exits.
-        elif fallback_storage is not None:
-            return DagsterInstance.from_config(fallback_storage)
+        if not dagster_home_path:
+            raise DagsterHomeNotSetError(
+                (
+                    "The environment variable $DAGSTER_HOME is not set. \n"
+                    "Dagster requires this environment variable to be set to an existing directory in your filesystem. "
+                    "This directory is used to store metadata across sessions, or load the dagster.yaml "
+                    "file which can configure storing metadata in an external database.\n"
+                    "You can resolve this error by exporting the environment variable. For example, you can run the following command in your shell or include it in your shell configuration file:\n"
+                    '\texport DAGSTER_HOME="~/dagster_home"\n'
+                    "Alternatively, DagsterInstance.ephemeral() can be used for a transient instance.\n"
+                )
+            )
 
-        # 3. If all else fails create an ephemeral in memory instance.
-        else:
-            return DagsterInstance.ephemeral(fallback_storage)
+        dagster_home_path = os.path.expanduser(dagster_home_path)
+
+        if not os.path.isabs(dagster_home_path):
+            raise DagsterInvariantViolationError(
+                (
+                    '$DAGSTER_HOME "{}" must be an absolute path. Dagster requires this '
+                    "environment variable to be set to an existing directory in your filesystem."
+                ).format(dagster_home_path)
+            )
+
+        if not (os.path.exists(dagster_home_path) and os.path.isdir(dagster_home_path)):
+            raise DagsterInvariantViolationError(
+                (
+                    '$DAGSTER_HOME "{}" is not a directory or does not exist. Dagster requires this '
+                    "environment variable to be set to an existing directory in your filesystem"
+                ).format(dagster_home_path)
+            )
+
+        return DagsterInstance.from_config(dagster_home_path)
 
     @staticmethod
     def local_temp(tempdir=None, overrides=None):
-        warnings.warn(
-            "To create a local DagsterInstance for a test, use the instance_for_test "
-            "context manager instead, which ensures that resoures are cleaned up afterwards"
-        )
-
         if tempdir is None:
             tempdir = DagsterInstance.temp_storage()
 
@@ -387,9 +372,7 @@ class DagsterInstance:
 
         # DagsterInstance doesn't implement ConfigurableClass, but we may still sometimes want to
         # have custom subclasses of DagsterInstance. This machinery allows for those custom
-        # subclasses to receive additional keyword arguments passed through the config YAML. Note
-        # that unlike a ConfigurableClass, these additional arguments are not type checked -- the
-        # raw Python dict returned by yaml.load is just splatted into kwargs.
+        # subclasses to receive additional keyword arguments passed through the config YAML.
         klass = instance_ref.custom_instance_class or DagsterInstance
         kwargs = instance_ref.custom_instance_class_config
 
@@ -560,6 +543,7 @@ class DagsterInstance:
             if print_fn:
                 print_fn("Updating event storage...")
             self._event_storage.upgrade()
+            self._event_storage.reindex_assets(print_fn=print_fn)
 
             if print_fn:
                 print_fn("Updating schedule storage...")
@@ -574,8 +558,8 @@ class DagsterInstance:
 
     def reindex(self, print_fn=lambda _: None):
         print_fn("Checking for reindexing...")
-        self._event_storage.reindex(print_fn)
-        self._run_storage.reindex(print_fn)
+        self._event_storage.reindex_events(print_fn)
+        self._event_storage.reindex_assets(print_fn)
         print_fn("Done.")
 
     def dispose(self):
@@ -641,8 +625,9 @@ class DagsterInstance:
         root_run_id=None,
         parent_run_id=None,
         solid_selection=None,
+        external_pipeline_origin=None,
+        pipeline_code_origin=None,
     ):
-        from dagster.core.execution.api import create_execution_plan
         from dagster.core.execution.plan.plan import ExecutionPlan
         from dagster.core.snap import snapshot_from_execution_plan
 
@@ -674,36 +659,48 @@ class DagsterInstance:
                     solids_to_execute=solids_to_execute
                 )
 
-        full_execution_plan = execution_plan or create_execution_plan(
-            pipeline_def,
-            run_config=run_config,
-            mode=mode,
-        )
-
-        if is_memoized_run(tags):
-            from dagster.core.execution.resolve_versions import resolve_memoized_execution_plan
-
-            if step_keys_to_execute:
-                raise DagsterInvariantViolationError(
-                    "step_keys_to_execute parameter cannot be used in conjunction with memoized "
-                    "pipeline runs."
-                )
-
-            subsetted_execution_plan = resolve_memoized_execution_plan(
-                full_execution_plan, run_config, self
-            )  # TODO: tighter integration with existing step_keys_to_execute functionality
-            step_keys_to_execute = subsetted_execution_plan.step_keys_to_execute
-            if not step_keys_to_execute:
-                raise DagsterNoStepsToExecuteException(
-                    "No steps found to execute. "
-                    "This is because every step in the plan has already been memoized."
-                )
-        else:
-            subsetted_execution_plan = (
-                full_execution_plan.build_subset_plan(step_keys_to_execute)
-                if step_keys_to_execute
-                else full_execution_plan
+        if execution_plan:
+            final_execution_plan = execution_plan
+            check.invariant(
+                step_keys_to_execute is None,
+                "Should not pass execution_plan and step_keys_to_execute to create_run",
             )
+            step_keys_to_execute = execution_plan.step_keys_to_execute
+        else:
+            resolved_run_config = ResolvedRunConfig.build(pipeline_def, run_config, mode)
+            full_execution_plan = ExecutionPlan.build(
+                InMemoryPipeline(pipeline_def),
+                resolved_run_config,
+            )
+
+            if is_memoized_run(tags):
+                from dagster.core.execution.resolve_versions import resolve_memoized_execution_plan
+
+                if step_keys_to_execute:
+                    raise DagsterInvariantViolationError(
+                        "step_keys_to_execute parameter cannot be used in conjunction with memoized "
+                        "pipeline runs."
+                    )
+
+                final_execution_plan = resolve_memoized_execution_plan(
+                    full_execution_plan,
+                    pipeline_def,
+                    run_config,
+                    self,
+                    resolved_run_config,
+                )  # TODO: tighter integration with existing step_keys_to_execute functionality
+                step_keys_to_execute = final_execution_plan.step_keys_to_execute
+                if not step_keys_to_execute:
+                    raise DagsterNoStepsToExecuteException(
+                        "No steps found to execute. "
+                        "This is because every step in the plan has already been memoized."
+                    )
+            elif step_keys_to_execute:
+                final_execution_plan = full_execution_plan.build_subset_plan(
+                    step_keys_to_execute, pipeline_def, resolved_run_config
+                )
+            else:
+                final_execution_plan = full_execution_plan
 
         return self.create_run(
             pipeline_name=pipeline_def.name,
@@ -719,9 +716,12 @@ class DagsterInstance:
             parent_run_id=parent_run_id,
             pipeline_snapshot=pipeline_def.get_pipeline_snapshot(),
             execution_plan_snapshot=snapshot_from_execution_plan(
-                subsetted_execution_plan, pipeline_def.get_pipeline_snapshot_id()
+                final_execution_plan,
+                pipeline_def.get_pipeline_snapshot_id(),
             ),
             parent_pipeline_snapshot=pipeline_def.get_parent_pipeline_snapshot(),
+            external_pipeline_origin=external_pipeline_origin,
+            pipeline_code_origin=pipeline_code_origin,
         )
 
     def _construct_run_with_snapshots(
@@ -741,6 +741,7 @@ class DagsterInstance:
         parent_pipeline_snapshot,
         solid_selection=None,
         external_pipeline_origin=None,
+        pipeline_code_origin=None,
     ):
 
         # https://github.com/dagster-io/dagster/issues/2403
@@ -784,6 +785,7 @@ class DagsterInstance:
             pipeline_snapshot_id=pipeline_snapshot_id,
             execution_plan_snapshot_id=execution_plan_snapshot_id,
             external_pipeline_origin=external_pipeline_origin,
+            pipeline_code_origin=pipeline_code_origin,
         )
 
     def _ensure_persisted_pipeline_snapshot(self, pipeline_snapshot, parent_pipeline_snapshot):
@@ -882,6 +884,7 @@ class DagsterInstance:
         parent_pipeline_snapshot,
         solid_selection=None,
         external_pipeline_origin=None,
+        pipeline_code_origin=None,
     ):
 
         pipeline_run = self._construct_run_with_snapshots(
@@ -900,6 +903,7 @@ class DagsterInstance:
             execution_plan_snapshot=execution_plan_snapshot,
             parent_pipeline_snapshot=parent_pipeline_snapshot,
             external_pipeline_origin=external_pipeline_origin,
+            pipeline_code_origin=pipeline_code_origin,
         )
         return self._run_storage.add_run(pipeline_run)
 
@@ -995,6 +999,27 @@ class DagsterInstance:
     ) -> Dict[str, Dict[str, Union[Iterable[PipelineRun], int]]]:
         return self._run_storage.get_run_groups(filters=filters, cursor=cursor, limit=limit)
 
+    def get_run_records(
+        self,
+        filters: PipelineRunsFilter = None,
+        limit: int = None,
+        order_by: str = None,
+        ascending: bool = False,
+    ) -> List[RunRecord]:
+        """Return a list of run records stored in the run storage, sorted by the given column in given order.
+
+        Args:
+            filters (Optional[PipelineRunsFilter]): the filter by which to filter runs.
+            limit (Optional[int]): Number of results to get. Defaults to infinite.
+            order_by (Optional[str]): Name of the column to sort by. Defaults to id.
+            ascending (Optional[bool]): Sort the result in ascending order if True, descending
+                otherwise. Defaults to descending.
+
+        Returns:
+            List[RunRecord]: List of run records stored in the run storage.
+        """
+        return self._run_storage.get_run_records(filters, limit, order_by, ascending)
+
     def wipe(self):
         self._run_storage.wipe()
         self._event_storage.wipe()
@@ -1005,15 +1030,18 @@ class DagsterInstance:
 
     # event storage
 
-    def logs_after(self, run_id, cursor):
-        return self._event_storage.get_logs_for_run(run_id, cursor=cursor)
+    def logs_after(self, run_id, cursor, of_type: "DagsterEventType" = None):
+        return self._event_storage.get_logs_for_run(run_id, cursor=cursor, of_type=of_type)
 
-    def all_logs(self, run_id):
-        return self._event_storage.get_logs_for_run(run_id)
+    def all_logs(self, run_id, of_type: "DagsterEventType" = None):
+        return self._event_storage.get_logs_for_run(run_id, of_type=of_type)
 
     def watch_event_logs(self, run_id, cursor, cb):
         return self._event_storage.watch(run_id, cursor, cb)
 
+    def end_watch_event_logs(self, run_id, cb):
+        return self._event_storage.end_watch(run_id, cb)
+
     # asset storage
 
     def all_asset_keys(self):
@@ -1022,6 +1050,26 @@ class DagsterInstance:
     def has_asset_key(self, asset_key: AssetKey) -> bool:
         return self._event_storage.has_asset_key(asset_key)
 
+    def get_event_records(
+        self,
+        event_records_filter: Optional["EventRecordsFilter"] = None,
+        limit: Optional[int] = None,
+        ascending: bool = False,
+    ) -> Iterable["EventLogRecord"]:
+        """Return a list of event records stored in the event log storage.
+
+        Args:
+            event_records_filter (Optional[EventRecordsFilter]): the filter by which to filter event
+                records.
+            limit (Optional[int]): Number of results to get. Defaults to infinite.
+            ascending (Optional[bool]): Sort the result in ascending order if True, descending
+                otherwise. Defaults to descending.
+
+        Returns:
+            List[EventLogRecord]: List of event log records stored in the event log storage.
+        """
+        return self._event_storage.get_event_records(event_records_filter, limit, ascending)
+
     def events_for_asset_key(
         self,
         asset_key,
@@ -1029,17 +1077,37 @@ class DagsterInstance:
         before_cursor=None,
         after_cursor=None,
         cursor=None,
+        before_timestamp=None,
         limit=None,
         ascending=False,
     ):
         check.inst_param(asset_key, "asset_key", AssetKey)
 
+        warnings.warn(
+            """
+The method `events_for_asset_key` on DagsterInstance has been deprecated as of `0.12.0` in favor of
+the method `get_event_records`. The method `get_event_records` takes in an `EventRecordsFilter`
+argument that allows for filtering by asset key and asset key partitions. The return value is a
+list of `EventLogRecord` objects, each of which contains a storage_id and an event log entry.
+
+Example:
+records = instance.get_event_records(
+    EventRecordsFilter(
+        asset_key=asset_key,
+        asset_partitions=partitions,
+        after_cursor=after_cursor,
+    ),
+)
+"""
+        )
+
         return self._event_storage.get_asset_events(
             asset_key,
             partitions,
             before_cursor,
             after_cursor,
             limit,
+            before_timestamp=before_timestamp,
             ascending=ascending,
             include_cursor=True,
             cursor=cursor,
@@ -1095,7 +1163,7 @@ class DagsterInstance:
         Report a EngineEvent that occurred outside of a pipeline execution context.
         """
         from dagster.core.events import EngineEventData, DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventRecord
+        from dagster.core.events.log import EventLogEntry
 
         check.class_param(cls, "cls")
         check.str_param(message, "message")
@@ -1120,7 +1188,7 @@ class DagsterInstance:
             message=message,
             event_specific_data=engine_event_data,
         )
-        event_record = EventRecord(
+        event_record = EventLogEntry(
             message=message,
             user_message=message,
             level=log_level,
@@ -1138,7 +1206,7 @@ class DagsterInstance:
     def report_run_canceling(self, run, message=None):
 
         from dagster.core.events import DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventRecord
+        from dagster.core.events.log import EventLogEntry
 
         check.inst_param(run, "run", PipelineRun)
         message = check.opt_str_param(
@@ -1152,7 +1220,7 @@ class DagsterInstance:
             message=message,
         )
 
-        event_record = EventRecord(
+        event_record = EventLogEntry(
             message=message,
             user_message="",
             level=logging.INFO,
@@ -1171,7 +1239,7 @@ class DagsterInstance:
         message=None,
     ):
         from dagster.core.events import DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventRecord
+        from dagster.core.events.log import EventLogEntry
 
         check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
 
@@ -1186,7 +1254,7 @@ class DagsterInstance:
             pipeline_name=pipeline_run.pipeline_name,
             message=message,
         )
-        event_record = EventRecord(
+        event_record = EventLogEntry(
             message=message,
             user_message=message,
             level=logging.ERROR,
@@ -1202,7 +1270,7 @@ class DagsterInstance:
 
     def report_run_failed(self, pipeline_run, message=None):
         from dagster.core.events import DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventRecord
+        from dagster.core.events.log import EventLogEntry
 
         check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
 
@@ -1217,7 +1285,7 @@ class DagsterInstance:
             pipeline_name=pipeline_run.pipeline_name,
             message=message,
         )
-        event_record = EventRecord(
+        event_record = EventLogEntry(
             message=message,
             user_message=message,
             level=logging.ERROR,
@@ -1247,7 +1315,7 @@ class DagsterInstance:
 
     # Runs coordinator
 
-    def submit_run(self, run_id, external_pipeline):
+    def submit_run(self, run_id, workspace: "IWorkspace") -> PipelineRun:
         """Submit a pipeline run to the coordinator.
 
         This method delegates to the ``RunCoordinator``, configured on the instance, and will
@@ -1262,6 +1330,8 @@ class DagsterInstance:
         """
 
         from dagster.core.host_representation import ExternalPipelineOrigin
+        from dagster.core.origin import PipelinePythonOrigin
+        from dagster.core.run_coordinator import SubmitRunContext
 
         run = self.get_run_by_id(run_id)
         check.inst(
@@ -1269,10 +1339,15 @@ class DagsterInstance:
             ExternalPipelineOrigin,
             "External pipeline origin must be set for submitted runs",
         )
+        check.inst(
+            run.pipeline_code_origin,
+            PipelinePythonOrigin,
+            "Python origin must be set for submitted runs",
+        )
 
         try:
             submitted_run = self._run_coordinator.submit_run(
-                run, external_pipeline=external_pipeline
+                SubmitRunContext(run, workspace=workspace)
             )
         except:
             from dagster.core.events import EngineEventData
@@ -1290,7 +1365,7 @@ class DagsterInstance:
 
     # Run launcher
 
-    def launch_run(self, run_id, external_pipeline):
+    def launch_run(self, run_id: str, workspace: "IWorkspace"):
         """Launch a pipeline run.
 
         This method is typically called using `instance.submit_run` rather than being invoked
@@ -1303,17 +1378,19 @@ class DagsterInstance:
         Args:
             run_id (str): The id of the run the launch.
         """
+        from dagster.core.launcher import LaunchRunContext
+
         run = self.get_run_by_id(run_id)
 
         from dagster.core.events import EngineEventData, DagsterEvent, DagsterEventType
-        from dagster.core.events.log import EventRecord
+        from dagster.core.events.log import EventLogEntry
 
         launch_started_event = DagsterEvent(
             event_type_value=DagsterEventType.PIPELINE_STARTING.value,
             pipeline_name=run.pipeline_name,
         )
 
-        event_record = EventRecord(
+        event_record = EventLogEntry(
             message="",
             user_message="",
             level=logging.INFO,
@@ -1329,7 +1406,7 @@ class DagsterInstance:
         run = self.get_run_by_id(run_id)
 
         try:
-            self._run_launcher.launch_run(run, external_pipeline=external_pipeline)
+            self._run_launcher.launch_run(LaunchRunContext(pipeline_run=run, workspace=workspace))
         except:
             error = serializable_error_info_from_exc_info(sys.exc_info())
             self.report_engine_event(
@@ -1363,7 +1440,7 @@ class DagsterInstance:
 
     def scheduler_debug_info(self):
         from dagster.core.scheduler import SchedulerDebugInfo
-        from dagster.core.definitions.job import JobType
+        from dagster.core.definitions.run_request import JobType
         from dagster.core.scheduler.job import JobStatus
 
         errors = []
@@ -1414,7 +1491,7 @@ class DagsterInstance:
 
     def start_sensor(self, external_sensor):
         from dagster.core.scheduler.job import JobState, JobStatus, SensorJobData
-        from dagster.core.definitions.job import JobType
+        from dagster.core.definitions.run_request import JobType
 
         job_state = self.get_job_state(external_sensor.get_external_origin_id())
 
