@@ -2,7 +2,7 @@ import logging
 from abc import abstractmethod
 from collections import defaultdict
 from datetime import datetime
-from typing import Optional
+from typing import Iterable, Optional
 
 import pendulum
 import sqlalchemy as db
@@ -18,8 +18,14 @@ from dagster.serdes.errors import DeserializationError
 from dagster.utils import datetime_as_float, utc_datetime_from_naive, utc_datetime_from_timestamp
 
 from ..pipeline_run import PipelineRunStatsSnapshot
-from .base import EventLogRecord, EventLogStorage, EventsCursor, extract_asset_events_cursor
-from .migration import REINDEX_DATA_MIGRATIONS
+from .base import (
+    EventLogRecord,
+    EventLogStorage,
+    EventRecordsFilter,
+    RunShardedEventsCursor,
+    extract_asset_events_cursor,
+)
+from .migration import ASSET_DATA_MIGRATIONS, ASSET_KEY_INDEX_COLS, EVENT_LOG_DATA_MIGRATIONS
 from .schema import AssetKeyTable, SecondaryIndexMigrationTable, SqlEventLogStorageTable
 
 
@@ -94,6 +100,7 @@ class SqlEventLogStorage(EventLogStorage):
         if not event.is_dagster_event or not event.dagster_event.asset_key:
             return
 
+        materialization = event.dagster_event.step_materialization_data.materialization
         # We switched to storing the entire event record of the last materialization instead of just
         # the AssetMaterialization object, so that we have access to metadata like timestamp,
         # pipeline, run_id, etc.
@@ -105,25 +112,52 @@ class SqlEventLogStorage(EventLogStorage):
         # to `last_materialization_event`, for clarity.  For now, we should do some back-compat.
         #
         # https://github.com/dagster-io/dagster/issues/3945
+        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
+            insert_statement = (
+                AssetKeyTable.insert().values(  # pylint: disable=no-value-for-parameter
+                    asset_key=event.dagster_event.asset_key.to_string(),
+                    last_materialization=serialize_dagster_namedtuple(event),
+                    last_materialization_timestamp=utc_datetime_from_timestamp(event.timestamp),
+                    last_run_id=event.run_id,
+                    tags=seven.json.dumps(materialization.tags) if materialization.tags else None,
+                )
+            )
+            update_statement = (
+                AssetKeyTable.update()
+                .values(  # pylint: disable=no-value-for-parameter
+                    last_materialization=serialize_dagster_namedtuple(event),
+                    last_materialization_timestamp=utc_datetime_from_timestamp(event.timestamp),
+                    last_run_id=event.run_id,
+                    tags=seven.json.dumps(materialization.tags) if materialization.tags else None,
+                )
+                .where(
+                    AssetKeyTable.c.asset_key == event.dagster_event.asset_key.to_string(),
+                )
+            )
+        else:
+            insert_statement = (
+                AssetKeyTable.insert().values(  # pylint: disable=no-value-for-parameter
+                    asset_key=event.dagster_event.asset_key.to_string(),
+                    last_materialization=serialize_dagster_namedtuple(event),
+                    last_run_id=event.run_id,
+                )
+            )
+            update_statement = (
+                AssetKeyTable.update()
+                .values(  # pylint: disable=no-value-for-parameter
+                    last_materialization=serialize_dagster_namedtuple(event),
+                    last_run_id=event.run_id,
+                )
+                .where(
+                    AssetKeyTable.c.asset_key == event.dagster_event.asset_key.to_string(),
+                )
+            )
 
         with self.index_connection() as conn:
             try:
-                conn.execute(
-                    AssetKeyTable.insert().values(  # pylint: disable=no-value-for-parameter
-                        asset_key=event.dagster_event.asset_key.to_string(),
-                        last_materialization=serialize_dagster_namedtuple(event),
-                    )
-                )
+                conn.execute(insert_statement)
             except db.exc.IntegrityError:
-                conn.execute(
-                    AssetKeyTable.update()  # pylint: disable=no-value-for-parameter
-                    .values(
-                        last_materialization=serialize_dagster_namedtuple(event),
-                    )
-                    .where(
-                        AssetKeyTable.c.asset_key == event.dagster_event.asset_key.to_string(),
-                    )
-                )
+                conn.execute(update_statement)
 
     def store_event(self, event):
         """Store an event corresponding to a pipeline run.
@@ -376,22 +410,28 @@ class SqlEventLogStorage(EventLogStorage):
             for step_key, value in by_step_key.items()
         ]
 
-    def reindex(self, print_fn=None, force=False):
-        """Call this method to run any data migrations, reindexing to build summary tables."""
-        # Should be overridden by SqliteEventLogStorage and other storages that shard based on
-        # run_id
-        for migration_name, migration_fn in REINDEX_DATA_MIGRATIONS.items():
-            if self.has_secondary_index(migration_name):
-                if not force:
-                    if print_fn:
-                        print_fn("Skipping already reindexed summary: {}".format(migration_name))
-                    continue
-            if print_fn:
-                print_fn("Starting reindex: {}".format(migration_name))
-            migration_fn()(self, print_fn)
-            self.enable_secondary_index(migration_name)
-            if print_fn:
-                print_fn("Finished reindexing: {}".format(migration_name))
+    def _apply_migration(self, migration_name, migration_fn, print_fn, force):
+        if self.has_secondary_index(migration_name):
+            if not force:
+                if print_fn:
+                    print_fn("Skipping already reindexed summary: {}".format(migration_name))
+                return
+        if print_fn:
+            print_fn("Starting reindex: {}".format(migration_name))
+        migration_fn()(self, print_fn)
+        self.enable_secondary_index(migration_name)
+        if print_fn:
+            print_fn("Finished reindexing: {}".format(migration_name))
+
+    def reindex_events(self, print_fn=None, force=False):
+        """Call this method to run any data migrations across the event_log table"""
+        for migration_name, migration_fn in EVENT_LOG_DATA_MIGRATIONS.items():
+            self._apply_migration(migration_name, migration_fn, print_fn, force)
+
+    def reindex_assets(self, print_fn=None, force=False):
+        """Call this method to run any data migrations across the asset_keys table"""
+        for migration_name, migration_fn in ASSET_DATA_MIGRATIONS.items():
+            self._apply_migration(migration_name, migration_fn, print_fn, force)
 
     def wipe(self):
         """Clears the event log storage."""
@@ -531,32 +571,105 @@ class SqlEventLogStorage(EventLogStorage):
                     .values(migration_completed=datetime.now())
                 )
 
-    def _add_cursor_limit_to_query(
+    def _apply_filter_to_query(
         self,
         query,
-        before_cursor,
-        after_cursor,
-        limit,
-        ascending=False,
-        before_timestamp=None,
+        event_records_filter=None,
+        asset_details=None,
+        apply_cursor_filters=True,
     ):
-        """ Helper function to deal with cursor/limit pagination args """
+        if not event_records_filter:
+            return query
 
-        if before_cursor:
-            before_query = db.select([SqlEventLogStorageTable.c.id]).where(
-                SqlEventLogStorageTable.c.id == before_cursor
-            )
-            query = query.where(SqlEventLogStorageTable.c.id < before_query)
-        if after_cursor:
-            after_query = db.select([SqlEventLogStorageTable.c.id]).where(
-                SqlEventLogStorageTable.c.id == after_cursor
-            )
-            query = query.where(SqlEventLogStorageTable.c.id > after_query)
-        if before_timestamp:
+        if event_records_filter.event_type:
             query = query.where(
-                SqlEventLogStorageTable.c.timestamp < datetime.utcfromtimestamp(before_timestamp)
+                SqlEventLogStorageTable.c.dagster_event_type
+                == event_records_filter.event_type.value
             )
 
+        if event_records_filter.asset_key:
+            query = query.where(
+                db.or_(
+                    SqlEventLogStorageTable.c.asset_key
+                    == event_records_filter.asset_key.to_string(),
+                    SqlEventLogStorageTable.c.asset_key
+                    == event_records_filter.asset_key.to_string(legacy=True),
+                )
+            )
+
+        if event_records_filter.asset_partitions:
+            query = query.where(
+                SqlEventLogStorageTable.c.partition.in_(event_records_filter.asset_partitions)
+            )
+
+        if asset_details and asset_details.last_wipe_timestamp:
+            query = query.where(
+                SqlEventLogStorageTable.c.timestamp
+                > datetime.utcfromtimestamp(asset_details.last_wipe_timestamp)
+            )
+
+        if apply_cursor_filters:
+            # allow the run-sharded sqlite implementation to disable this cursor filtering so that
+            # it can implement its own custom cursor logic, as cursor ids are not unique across run
+            # shards
+            if event_records_filter.before_cursor is not None:
+                before_cursor_id = (
+                    event_records_filter.before_cursor.id
+                    if isinstance(event_records_filter.before_cursor, RunShardedEventsCursor)
+                    else event_records_filter.before_cursor
+                )
+                before_query = db.select([SqlEventLogStorageTable.c.id]).where(
+                    SqlEventLogStorageTable.c.id == before_cursor_id
+                )
+                query = query.where(SqlEventLogStorageTable.c.id < before_query)
+
+            if event_records_filter.after_cursor is not None:
+                after_cursor_id = (
+                    event_records_filter.after_cursor.id
+                    if isinstance(event_records_filter.after_cursor, RunShardedEventsCursor)
+                    else event_records_filter.after_cursor
+                )
+                after_query = db.select([SqlEventLogStorageTable.c.id]).where(
+                    SqlEventLogStorageTable.c.id == after_cursor_id
+                )
+                query = query.where(SqlEventLogStorageTable.c.id > after_query)
+
+        if event_records_filter.before_timestamp:
+            query = query.where(
+                SqlEventLogStorageTable.c.timestamp
+                < datetime.utcfromtimestamp(event_records_filter.before_timestamp)
+            )
+
+        if event_records_filter.after_timestamp:
+            query = query.where(
+                SqlEventLogStorageTable.c.timestamp
+                > datetime.utcfromtimestamp(event_records_filter.after_timestamp)
+            )
+
+        return query
+
+    def get_event_records(
+        self,
+        event_records_filter: Optional[EventRecordsFilter] = None,
+        limit: Optional[int] = None,
+        ascending: bool = False,
+    ) -> Iterable[EventLogRecord]:
+        """Returns a list of (record_id, record)."""
+        check.opt_inst_param(event_records_filter, "event_records_filter", EventRecordsFilter)
+        check.opt_int_param(limit, "limit")
+        check.bool_param(ascending, "ascending")
+
+        query = db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event])
+        if event_records_filter and event_records_filter.asset_key:
+            asset_details = self._get_asset_details(event_records_filter.asset_key)
+        else:
+            asset_details = None
+
+        query = self._apply_filter_to_query(
+            query=query,
+            event_records_filter=event_records_filter,
+            asset_details=asset_details,
+        )
         if limit:
             query = query.limit(limit)
 
@@ -565,36 +678,6 @@ class SqlEventLogStorage(EventLogStorage):
         else:
             query = query.order_by(SqlEventLogStorageTable.c.timestamp.desc())
 
-        return query
-
-    def get_event_records(
-        self,
-        after_cursor=None,
-        limit=None,
-        ascending=False,
-        of_type=None,
-    ):
-        """Returns a list of (record_id, record)."""
-
-        check.opt_inst_param(after_cursor, "after_cursor", EventsCursor)
-        check.opt_int_param(limit, "limit")
-        check.bool_param(ascending, "ascending")
-        check.opt_inst_param(of_type, "of_type", DagsterEventType)
-
-        after_id = after_cursor.id if after_cursor else None
-
-        query = db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event])
-        if of_type:
-            query = query.where(SqlEventLogStorageTable.c.dagster_event_type == of_type.value)
-
-        query = self._add_cursor_limit_to_query(
-            query=query,
-            before_cursor=None,
-            after_cursor=after_id,
-            limit=limit,
-            ascending=ascending,
-        )
-
         with self.index_connection() as conn:
             results = conn.execute(query).fetchall()
 
@@ -743,58 +826,31 @@ class SqlEventLogStorage(EventLogStorage):
         after_cursor=None,
         limit=None,
         ascending=False,
-        include_cursor=False,
+        include_cursor=False,  # deprecated
         before_timestamp=None,
         cursor=None,  # deprecated
     ):
         check.inst_param(asset_key, "asset_key", AssetKey)
         check.opt_list_param(partitions, "partitions", of_type=str)
-        query = db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event]).where(
-            db.or_(
-                SqlEventLogStorageTable.c.asset_key == asset_key.to_string(),
-                SqlEventLogStorageTable.c.asset_key == asset_key.to_string(legacy=True),
-            )
-        )
-        if partitions:
-            query = query.where(SqlEventLogStorageTable.c.partition.in_(partitions))
-
-        asset_details = self._get_asset_details(asset_key)
-        query = self._add_asset_wipe_filter_to_query(query, asset_details)
-
         before_cursor, after_cursor = extract_asset_events_cursor(
             cursor, before_cursor, after_cursor, ascending
         )
-
-        query = self._add_cursor_limit_to_query(
-            query,
-            before_cursor,
-            after_cursor,
-            limit,
-            before_timestamp=before_timestamp,
+        event_records = self.get_event_records(
+            EventRecordsFilter(
+                event_type=DagsterEventType.ASSET_MATERIALIZATION,
+                asset_key=asset_key,
+                asset_partitions=partitions,
+                before_cursor=before_cursor,
+                after_cursor=after_cursor,
+                before_timestamp=before_timestamp,
+            ),
+            limit=limit,
             ascending=ascending,
         )
-
-        with self.index_connection() as conn:
-            results = conn.execute(query).fetchall()
-
-        events = []
-        for row_id, json_str in results:
-            try:
-                event_record = deserialize_json_to_dagster_namedtuple(json_str)
-                if not isinstance(event_record, EventLogEntry):
-                    logging.warning(
-                        "Could not resolve asset event record as EventLogEntry for id `{}`.".format(
-                            row_id
-                        )
-                    )
-                    continue
-                if include_cursor:
-                    events.append(tuple([row_id, event_record]))
-                else:
-                    events.append(event_record)
-            except seven.JSONDecodeError:
-                logging.warning("Could not parse asset event record id `{}`.".format(row_id))
-        return events
+        if include_cursor:
+            return [tuple([record.storage_id, record.event_log_entry]) for record in event_records]
+        else:
+            return [record.event_log_entry for record in event_records]
 
     def get_asset_run_ids(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
@@ -852,49 +908,95 @@ class SqlEventLogStorage(EventLogStorage):
         return event_or_materialization.dagster_event.step_materialization_data.materialization
 
     def all_asset_tags(self):
-        query = db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.last_materialization])
         tags_by_asset_key = defaultdict(dict)
-        with self.index_connection() as conn:
-            rows = conn.execute(query).fetchall()
-            for asset_key, json_str in rows:
-                materialization = self._asset_materialization_from_json_column(json_str)
-                if materialization and materialization.tags:
-                    tags_by_asset_key[AssetKey.from_db_string(asset_key)] = {
-                        k: v for k, v in materialization.tags.items()
-                    }
+        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
+            query = db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.tags]).where(
+                AssetKeyTable.c.tags != None
+            )
+            with self.index_connection() as conn:
+                rows = conn.execute(query).fetchall()
+                for asset_key, tags_json in rows:
+                    tags = seven.json.loads(tags_json)
+                    if tags:
+                        tags_by_asset_key[AssetKey.from_db_string(asset_key)] = tags
+
+        else:
+            query = db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.last_materialization])
+            with self.index_connection() as conn:
+                rows = conn.execute(query).fetchall()
+                for asset_key, json_str in rows:
+                    materialization = self._asset_materialization_from_json_column(json_str)
+                    if materialization and materialization.tags:
+                        tags_by_asset_key[AssetKey.from_db_string(asset_key)] = {
+                            k: v for k, v in materialization.tags.items()
+                        }
 
         return tags_by_asset_key
 
     def get_asset_tags(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
-        query = db.select([AssetKeyTable.c.last_materialization]).where(
-            AssetKeyTable.c.asset_key == asset_key.to_string()
-        )
-        with self.index_connection() as conn:
-            rows = conn.execute(query).fetchall()
-            if not rows or not rows[0] or not rows[0][0]:
-                return {}
+        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
+            query = db.select([AssetKeyTable.c.tags]).where(
+                AssetKeyTable.c.asset_key == asset_key.to_string()
+            )
+            with self.index_connection() as conn:
+                rows = conn.execute(query).fetchall()
+                if not rows or not rows[0] or not rows[0][0]:
+                    return {}
 
-            materialization = self._asset_materialization_from_json_column(rows[0][0])
-            return materialization.tags if materialization and materialization.tags else {}
+                return seven.json.loads(rows[0][0])
+        else:
+            query = db.select([AssetKeyTable.c.last_materialization]).where(
+                AssetKeyTable.c.asset_key == asset_key.to_string()
+            )
+            with self.index_connection() as conn:
+                rows = conn.execute(query).fetchall()
+                if not rows or not rows[0] or not rows[0][0]:
+                    return {}
+
+                materialization = self._asset_materialization_from_json_column(rows[0][0])
+                return materialization.tags if materialization and materialization.tags else {}
 
     def wipe_asset(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
 
-        with self.index_connection() as conn:
-            conn.execute(
-                AssetKeyTable.update()  # pylint: disable=no-value-for-parameter
-                .where(
-                    db.or_(
-                        AssetKeyTable.c.asset_key == asset_key.to_string(),
-                        AssetKeyTable.c.asset_key == asset_key.to_string(legacy=True),
+        wipe_timestamp = pendulum.now("UTC").timestamp()
+
+        if self.has_secondary_index(ASSET_KEY_INDEX_COLS):
+            with self.index_connection() as conn:
+                conn.execute(
+                    AssetKeyTable.update()  # pylint: disable=no-value-for-parameter
+                    .where(
+                        db.or_(
+                            AssetKeyTable.c.asset_key == asset_key.to_string(),
+                            AssetKeyTable.c.asset_key == asset_key.to_string(legacy=True),
+                        )
+                    )
+                    .values(
+                        last_materialization=None,
+                        last_run_id=None,
+                        asset_details=serialize_dagster_namedtuple(
+                            AssetDetails(last_wipe_timestamp=wipe_timestamp)
+                        ),
+                        wipe_timestamp=utc_datetime_from_timestamp(wipe_timestamp),
                     )
                 )
-                .values(
-                    last_materialization=None,
-                    last_run_id=None,
-                    asset_details=serialize_dagster_namedtuple(
-                        AssetDetails(last_wipe_timestamp=pendulum.now("UTC").timestamp())
-                    ),
+
+        else:
+            with self.index_connection() as conn:
+                conn.execute(
+                    AssetKeyTable.update()  # pylint: disable=no-value-for-parameter
+                    .where(
+                        db.or_(
+                            AssetKeyTable.c.asset_key == asset_key.to_string(),
+                            AssetKeyTable.c.asset_key == asset_key.to_string(legacy=True),
+                        )
+                    )
+                    .values(
+                        last_materialization=None,
+                        last_run_id=None,
+                        asset_details=serialize_dagster_namedtuple(
+                            AssetDetails(last_wipe_timestamp=pendulum.now("UTC").timestamp())
+                        ),
+                    )
                 )
-            )
