@@ -5,6 +5,7 @@ from enum import Enum
 from typing import NamedTuple
 
 from dagster import check
+from dagster.core.origin import PipelinePythonOrigin
 from dagster.core.storage.tags import PARENT_RUN_ID_TAG, ROOT_RUN_ID_TAG
 from dagster.core.utils import make_new_run_id
 from dagster.serdes import DefaultNamedTupleSerializer, whitelist_for_serdes
@@ -117,6 +118,7 @@ def pipeline_run_from_storage(
     solid_subset=None,
     reexecution_config=None,  # pylint: disable=unused-argument
     external_pipeline_origin=None,
+    pipeline_code_origin=None,
     **kwargs,
 ):
 
@@ -197,6 +199,7 @@ def pipeline_run_from_storage(
         pipeline_snapshot_id=pipeline_snapshot_id,
         execution_plan_snapshot_id=execution_plan_snapshot_id,
         external_pipeline_origin=external_pipeline_origin,
+        pipeline_code_origin=pipeline_code_origin,
     )
 
 
@@ -207,7 +210,8 @@ class PipelineRun(
         (
             "pipeline_name run_id run_config mode solid_selection solids_to_execute "
             "step_keys_to_execute status tags root_run_id parent_run_id "
-            "pipeline_snapshot_id execution_plan_snapshot_id external_pipeline_origin"
+            "pipeline_snapshot_id execution_plan_snapshot_id external_pipeline_origin "
+            "pipeline_code_origin"
         ),
     )
 ):
@@ -230,7 +234,12 @@ class PipelineRun(
         parent_run_id=None,
         pipeline_snapshot_id=None,
         execution_plan_snapshot_id=None,
+        # An ExternalPipelineOrigin that can be used to recreate the RepositoryLocation and
+        # ExternalPipeline that was used to submit this run
         external_pipeline_origin=None,
+        # A PipelinePythonOrigin with information about where to find the pipeline definition in
+        # code. Most run launchers will pass this origin as an argument to the run worker process.
+        pipeline_code_origin=None,
     ):
         check.invariant(
             (root_run_id is not None and parent_run_id is not None)
@@ -281,6 +290,9 @@ class PipelineRun(
             external_pipeline_origin=check.opt_inst_param(
                 external_pipeline_origin, "external_pipeline_origin", ExternalPipelineOrigin
             ),
+            pipeline_code_origin=check.opt_inst_param(
+                pipeline_code_origin, "pipeline_code_origin", PipelinePythonOrigin
+            ),
         )
 
     def with_status(self, status):
@@ -354,7 +366,7 @@ class PipelineRun(
 @whitelist_for_serdes
 class PipelineRunsFilter(
     namedtuple(
-        "_PipelineRunsFilter", "run_ids pipeline_name statuses tags snapshot_id updated_after"
+        "_PipelineRunsFilter", "run_ids pipeline_name statuses tags snapshot_id updated_after mode"
     )
 ):
     def __new__(
@@ -365,6 +377,7 @@ class PipelineRunsFilter(
         tags=None,
         snapshot_id=None,
         updated_after=None,
+        mode=None,
     ):
         return super(PipelineRunsFilter, cls).__new__(
             cls,
@@ -374,6 +387,7 @@ class PipelineRunsFilter(
             tags=check.opt_dict_param(tags, "tags", key_type=str, value_type=str),
             snapshot_id=check.opt_str_param(snapshot_id, "snapshot_id"),
             updated_after=check.opt_inst_param(updated_after, "updated_after", datetime),
+            mode=check.opt_str_param(mode, "mode"),
         )
 
     @staticmethod
