@@ -51,7 +51,7 @@ def assert_correct_extra_repository_output(result):
 
 @pytest.mark.skipif(seven.IS_WINDOWS, reason="no named sockets on Windows")
 def test_list_command_grpc_socket():
-    with instance_for_test() as instance:
+    with instance_for_test():
         runner = CliRunner()
 
         server_process = GrpcServerProcess(
@@ -66,12 +66,10 @@ def test_list_command_grpc_socket():
             execute_list_command(
                 {"grpc_socket": api_client.socket},
                 no_print,
-                instance,
             )
             execute_list_command(
                 {"grpc_socket": api_client.socket, "grpc_host": api_client.host},
                 no_print,
-                instance,
             )
 
             result = runner.invoke(pipeline_list_command, ["--grpc-socket", api_client.socket])
@@ -87,9 +85,9 @@ def test_list_command_grpc_socket():
 
 
 def test_list_command_deployed_grpc():
-    runner = CliRunner()
+    with instance_for_test():
+        runner = CliRunner()
 
-    with instance_for_test() as instance:
         server_process = GrpcServerProcess(
             loadable_target_origin=LoadableTargetOrigin(
                 executable_path=sys.executable,
@@ -121,7 +119,6 @@ def test_list_command_deployed_grpc():
             execute_list_command(
                 {"grpc_port": api_client.port},
                 no_print,
-                instance,
             )
 
             # Can't supply both port and socket
@@ -129,7 +126,6 @@ def test_list_command_deployed_grpc():
                 execute_list_command(
                     {"grpc_port": api_client.port, "grpc_socket": "foonamedsocket"},
                     no_print,
-                    instance,
                 )
 
         server_process.wait()
@@ -207,7 +203,7 @@ def test_list_command_cli():
 
 
 def test_list_command():
-    with instance_for_test() as instance:
+    with instance_for_test():
         execute_list_command(
             {
                 "repository_yaml": None,
@@ -216,7 +212,6 @@ def test_list_command():
                 "fn_name": "bar",
             },
             no_print,
-            instance,
         )
 
         execute_list_command(
@@ -228,7 +223,6 @@ def test_list_command():
                 "working_directory": os.path.dirname(__file__),
             },
             no_print,
-            instance,
         )
 
         execute_list_command(
@@ -239,7 +233,6 @@ def test_list_command():
                 "fn_name": "bar",
             },
             no_print,
-            instance,
         )
 
         with pytest.raises(UsageError):
@@ -251,5 +244,4 @@ def test_list_command():
                     "fn_name": "bar",
                 },
                 no_print,
-                instance,
             )
