@@ -1,9 +1,10 @@
-from typing import Dict
-
-import pytest
-from dagster import ConfigMapping, DagsterInvalidDefinitionError, execute_pipeline, resource, solid
-from dagster.core.definitions.decorators.graph import graph
+from dagster import ConfigMapping, graph, logger, resource, solid, success_hook
 from dagster.core.definitions.graph import GraphDefinition
+from dagster.core.definitions.partition import (
+    Partition,
+    PartitionedConfig,
+    StaticPartitionsDefinition,
+)
 from dagster.core.execution.execute import execute_in_process
 
 
@@ -63,39 +64,10 @@ def test_with_resources():
     # proxy for "executable/job"
     my_job = my_graph.to_job(resource_defs={"a": a_resource})
     assert my_job.name == "my_graph"
-    result = execute_pipeline(my_job)
+    result = my_job.execute_in_process()
     assert result.success
 
 
-def test_config_mapping_val():
-    @resource(config_schema=str)
-    def date(context) -> str:
-        return context.resource_config
-
-    @solid(
-        required_resource_keys={"date"},
-        config_schema={"msg": str},
-    )
-    def do_stuff(context):
-        return f"{context.solid_config['msg'] } on {context.resources.date}"
-
-    @graph
-    def needs_config():
-        do_stuff()
-
-    job = needs_config.to_job(
-        resource_defs={"date": date},
-        config_mapping={
-            "solids": {"do_stuff": {"config": {"msg": "i am here"}}},
-            "resources": {"date": {"config": "6/3"}},
-        },
-    )
-
-    result = execute_pipeline(job)
-    assert result.success
-    assert result.result_for_solid("do_stuff").output_value() == "i am here on 6/3"
-
-
 def test_config_mapping_fn():
     @resource(config_schema=str)
     def date(context) -> str:
@@ -120,15 +92,15 @@ def test_config_mapping_fn():
 
     job = needs_config.to_job(
         resource_defs={"date": date},
-        config_mapping=ConfigMapping(
+        config=ConfigMapping(
             config_schema={"date": str},  # top level has to be dict
             config_fn=_mapped,
         ),
     )
 
-    result = execute_pipeline(job, run_config={"date": "6/4"})
+    result = job.execute_in_process(run_config={"date": "6/4"})
     assert result.success
-    assert result.result_for_solid("do_stuff").output_value() == "i am here on 6/4"
+    assert result.result_for_node("do_stuff").output_values["result"] == "i am here on 6/4"
 
 
 def test_default_config():
@@ -149,51 +121,15 @@ def test_default_config():
 
     job = needs_config.to_job(
         resource_defs={"date": date},
-        default_config={
+        config={
             "solids": {"do_stuff": {"config": {"msg": "i am here"}}},
             "resources": {"date": {"config": "6/3"}},
         },
     )
 
-    result = execute_pipeline(job)
+    result = job.execute_in_process()
     assert result.success
-    assert result.result_for_solid("do_stuff").output_value() == "i am here on 6/3"
-
-
-def test_default_config_with_mapping_fn():
-    @resource(config_schema=str)
-    def date(context) -> str:
-        return context.resource_config
-
-    @solid(
-        required_resource_keys={"date"},
-        config_schema={"msg": str},
-    )
-    def do_stuff(context):
-        return f"{context.solid_config['msg'] } on {context.resources.date}"
-
-    @graph
-    def needs_config():
-        do_stuff()
-
-    def _mapped(val):
-        return {
-            "solids": {"do_stuff": {"config": {"msg": "i am here"}}},
-            "resources": {"date": {"config": val["date"]}},
-        }
-
-    job = needs_config.to_job(
-        resource_defs={"date": date},
-        config_mapping=ConfigMapping(
-            config_schema={"date": str},  # top level has to be dict
-            config_fn=_mapped,
-        ),
-        default_config={"date": "6/4"},
-    )
-
-    result = execute_pipeline(job)
-    assert result.success
-    assert result.result_for_solid("do_stuff").output_value() == "i am here on 6/4"
+    assert result.result_for_node("do_stuff").output_values["result"] == "i am here on 6/3"
 
 
 def test_suffix():
@@ -218,67 +154,26 @@ def test_partitions():
     def my_graph():
         my_solid()
 
-    def config_fn(date_str: str):
-        return {"solids": {"my_solid": {"config": {"date": date_str}}}}
+    def config_fn(partition: Partition):
+        return {"solids": {"my_solid": {"config": {"date": partition.value}}}}
 
-    def partition_fn():
-        return ["2020-02-25", "2020-02-26"]
-
-    job = my_graph.to_job(config_mapping=ConfigMapping(config_fn), partitions=partition_fn)
+    job = my_graph.to_job(
+        config=PartitionedConfig(
+            run_config_for_partition_fn=config_fn,
+            partitions_def=StaticPartitionsDefinition(
+                [Partition("2020-02-25"), Partition("2020-02-26")]
+            ),
+        ),
+    )
     mode = job.mode_definitions[0]
     partition_set = mode.get_partition_set_def("my_graph")
     partitions = partition_set.get_partitions()
     assert len(partitions) == 2
     assert partitions[0].value == "2020-02-25"
     assert partitions[0].name == "2020-02-25"
-    assert partition_set.run_config_for_partition(partitions[0]) == "2020-02-25"
-
-
-def test_non_str_partitions():
-    @solid(config_schema={"date": str})
-    def my_solid(_):
-        pass
-
-    @graph
-    def my_graph():
-        my_solid()
-
-    def config_fn(date_blob: Dict[str, str]):
-        return {"solids": {"my_solid": {"config": {"date": date_blob["date"]}}}}
-
-    def partition_fn():
-        return [{"date": "2020-02-25"}, {"date": "2020-02-26"}]
-
-    job = my_graph.to_job(config_mapping=ConfigMapping(config_fn), partitions=partition_fn)
-    mode = job.mode_definitions[0]
-    partition_set = mode.get_partition_set_def("my_graph")
-    partitions = partition_set.get_partitions()
-    assert len(partitions) == 2
-    assert partitions[0].value == {"date": "2020-02-25"}
-    assert partitions[0].name == str({"date": "2020-02-25"})
-    assert partition_set.run_config_for_partition(partitions[0]) == {"date": "2020-02-25"}
-
-
-def test_partitions_and_default_config():
-    @solid(config_schema={"date": str})
-    def my_solid(_):
-        pass
-
-    @graph
-    def my_graph():
-        my_solid()
-
-    def partition_fn():
-        return []
-
-    with pytest.raises(
-        DagsterInvalidDefinitionError,
-        match="A job can have default_config or partitions, but not both",
-    ):
-        my_graph.to_job(
-            partitions=partition_fn,
-            default_config={"solids": {"my_solid": {"config": {"date": "abc"}}}},
-        )
+    assert partition_set.run_config_for_partition(partitions[0]) == {
+        "solids": {"my_solid": {"config": {"date": "2020-02-25"}}}
+    }
 
 
 def test_tags_on_job():
@@ -294,5 +189,45 @@ def test_tags_on_job():
     job = basic_graph.to_job(tags=tags)
     assert job.tags == tags
 
-    result = execute_pipeline(job)
+    result = job.execute_in_process()
     assert result.success
+
+
+def test_logger_defs():
+    @solid
+    def my_solid(_):
+        pass
+
+    @graph
+    def my_graph():
+        my_solid()
+
+    @logger
+    def my_logger(_):
+        pass
+
+    my_job = my_graph.to_job(logger_defs={"abc": my_logger})
+    assert my_job.mode_definitions[0].loggers == {"abc": my_logger}
+
+
+def test_job_with_hooks():
+    entered = []
+
+    @success_hook
+    def basic_hook(_):
+        entered.append("yes")
+
+    @solid
+    def basic_emit():
+        pass
+
+    @graph
+    def basic_hook_graph():
+        basic_emit()
+
+    job_for_hook_testing = basic_hook_graph.to_job(hooks={basic_hook})
+
+    result = job_for_hook_testing.execute_in_process()
+
+    assert result.success
+    assert entered == ["yes"]
