@@ -2,6 +2,7 @@ import os
 import random
 import string
 import sys
+import tempfile
 import time
 
 import pytest
@@ -20,6 +21,7 @@ from dagster.core.instance import DagsterInstance
 from dagster.core.storage.compute_log_manager import ComputeIOType
 from dagster.core.test_utils import create_run_for_test, instance_for_test
 from dagster.seven import multiprocessing
+from dagster.utils import ensure_dir, touch_file
 
 HELLO_SOLID = "HELLO SOLID"
 HELLO_RESOURCE = "HELLO RESOURCE"
@@ -164,6 +166,44 @@ def test_compute_log_manager_subscriptions():
         assert stderr[0].cursor > 400
 
 
+@pytest.mark.skipif(
+    should_disable_io_stream_redirect(), reason="compute logs disabled for win / py3.6+"
+)
+def test_compute_log_manager_subscription_updates():
+    from dagster.core.storage.local_compute_log_manager import LocalComputeLogManager
+
+    with tempfile.TemporaryDirectory() as temp_dir:
+        compute_log_manager = LocalComputeLogManager(temp_dir, polling_timeout=0.5)
+        run_id = "fake_run_id"
+        step_key = "spew"
+        stdout_path = compute_log_manager.get_local_path(run_id, step_key, ComputeIOType.STDOUT)
+
+        # make sure the parent directory to be watched exists, file exists
+        ensure_dir(os.path.dirname(stdout_path))
+        touch_file(stdout_path)
+
+        # set up the subscription
+        messages = []
+        observable = compute_log_manager.observable(run_id, step_key, ComputeIOType.STDOUT)
+        observable.subscribe(messages.append)
+
+        # returns a single update, with 0 data
+        assert len(messages) == 1
+        last_chunk = messages[-1]
+        assert not last_chunk.data
+        assert last_chunk.cursor == 0
+
+        with open(stdout_path, "a+") as f:
+            print(HELLO_SOLID, file=f)  # pylint:disable=print-call
+
+        # wait longer than the watchdog timeout
+        time.sleep(1)
+        assert len(messages) == 2
+        last_chunk = messages[-1]
+        assert last_chunk.data
+        assert last_chunk.cursor > 0
+
+
 def gen_solid_name(length):
     return "".join(random.choice(string.ascii_lowercase) for x in range(length))
 
@@ -257,6 +297,47 @@ def test_single():
         assert normalize_file_content(full_out.data).startswith(expected_outer_prefix())
 
 
+@pytest.mark.skipif(
+    should_disable_io_stream_redirect(), reason="compute logs disabled for win / py3.6+"
+)
+def test_compute_log_base_with_spaces():
+    with tempfile.TemporaryDirectory() as temp_dir:
+        with instance_for_test(
+            temp_dir=temp_dir,
+            overrides={
+                "compute_logs": {
+                    "module": "dagster.core.storage.local_compute_log_manager",
+                    "class": "LocalComputeLogManager",
+                    "config": {"base_dir": os.path.join(temp_dir, "base with spaces")},
+                }
+            },
+        ) as instance:
+            pipeline_name = "foo_pipeline"
+            pipeline_run = create_run_for_test(instance, pipeline_name=pipeline_name)
+
+            step_keys = ["A", "B", "C"]
+
+            with instance.compute_log_manager.watch(pipeline_run):
+                print("outer 1")  # pylint: disable=print-call
+                print("outer 2")  # pylint: disable=print-call
+                print("outer 3")  # pylint: disable=print-call
+
+                for step_key in step_keys:
+                    inner_step(instance, pipeline_run, step_key)
+
+            for step_key in step_keys:
+                stdout = instance.compute_log_manager.read_logs_file(
+                    pipeline_run.run_id, step_key, ComputeIOType.STDOUT
+                )
+                assert normalize_file_content(stdout.data) == expected_inner_output(step_key)
+
+            full_out = instance.compute_log_manager.read_logs_file(
+                pipeline_run.run_id, pipeline_name, ComputeIOType.STDOUT
+            )
+
+            assert normalize_file_content(full_out.data).startswith(expected_outer_prefix())
+
+
 @pytest.mark.skipif(
     should_disable_io_stream_redirect(), reason="compute logs disabled for win / py3.6+"
 )
