@@ -1,4 +1,6 @@
 import os
+import random
+import string
 import sys
 import threading
 import time
@@ -6,28 +8,42 @@ from contextlib import contextmanager
 
 import pendulum
 import pytest
-from dagster import pipeline, repository, solid
-from dagster.core.definitions.decorators.sensor import sensor
-from dagster.core.definitions.job import JobType
+from dagster import (
+    Any,
+    AssetKey,
+    AssetMaterialization,
+    Field,
+    Output,
+    graph,
+    pipeline,
+    repository,
+    solid,
+)
+from dagster.core.definitions.decorators.sensor import asset_sensor, sensor
 from dagster.core.definitions.reconstructable import ReconstructableRepository
-from dagster.core.definitions.sensor import RunRequest, SkipReason
+from dagster.core.definitions.run_request import JobType
+from dagster.core.definitions.sensor import DEFAULT_SENSOR_DAEMON_INTERVAL, RunRequest, SkipReason
 from dagster.core.execution.api import execute_pipeline
 from dagster.core.host_representation import (
     ExternalJobOrigin,
     ExternalRepositoryOrigin,
     InProcessRepositoryLocationOrigin,
     ManagedGrpcPythonEnvRepositoryLocationOrigin,
-    RepositoryLocationHandleManager,
 )
 from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
 from dagster.core.scheduler.job import JobState, JobStatus, JobTickStatus
 from dagster.core.storage.pipeline_run import PipelineRunStatus
 from dagster.core.test_utils import instance_for_test
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
+from dagster.core.workspace.dynamic_workspace import DynamicWorkspace
 from dagster.daemon import get_default_daemon_logger
-from dagster.daemon.daemon import SensorDaemon
+from dagster.daemon.controller import (
+    DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
+    DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
+)
+from dagster.daemon.daemon import DAEMON_HEARTBEAT_ERROR_LIMIT, SensorDaemon
 from dagster.daemon.sensor import execute_sensor_iteration, execute_sensor_iteration_loop
-from dagster.seven import create_pendulum_time, to_timezone
+from dagster.seven.compat.pendulum import create_pendulum_time, to_timezone
 
 
 @solid
@@ -40,6 +56,35 @@ def the_pipeline():
     the_solid()
 
 
+@graph
+def the_graph():
+    the_solid()
+
+
+the_job = the_graph.to_job()
+
+
+@solid(config_schema=Field(Any))
+def config_solid(_):
+    return 1
+
+
+@pipeline
+def config_pipeline():
+    config_solid()
+
+
+@solid
+def foo_solid():
+    yield AssetMaterialization(asset_key=AssetKey("foo"))
+    yield Output(1)
+
+
+@pipeline
+def foo_pipeline():
+    foo_solid()
+
+
 @sensor(pipeline_name="the_pipeline")
 def simple_sensor(context):
     if not context.last_completion_time or not int(context.last_completion_time) % 2:
@@ -73,16 +118,76 @@ def custom_interval_sensor(_context):
     return SkipReason()
 
 
+@sensor(pipeline_name="the_pipeline")
+def skip_cursor_sensor(context):
+    if not context.cursor:
+        cursor = 1
+    else:
+        cursor = int(context.cursor) + 1
+
+    context.update_cursor(str(cursor))
+    return SkipReason()
+
+
+@sensor(pipeline_name="the_pipeline")
+def run_cursor_sensor(context):
+    if not context.cursor:
+        cursor = 1
+    else:
+        cursor = int(context.cursor) + 1
+
+    context.update_cursor(str(cursor))
+    return RunRequest(run_key=None, run_config={}, tags={})
+
+
+def _random_string(length):
+    return "".join(random.choice(string.ascii_lowercase) for x in range(length))
+
+
+@sensor(pipeline_name="config_pipeline")
+def large_sensor(_context):
+    # create a gRPC response payload larger than the limit (4194304)
+    REQUEST_COUNT = 25
+    REQUEST_TAG_COUNT = 5000
+    REQUEST_CONFIG_COUNT = 100
+
+    for _ in range(REQUEST_COUNT):
+        tags_garbage = {_random_string(10): _random_string(20) for i in range(REQUEST_TAG_COUNT)}
+        config_garbage = {
+            _random_string(10): _random_string(20) for i in range(REQUEST_CONFIG_COUNT)
+        }
+        config = {"solids": {"config_solid": {"config": {"foo": config_garbage}}}}
+        yield RunRequest(run_key=None, run_config=config, tags=tags_garbage)
+
+
+@asset_sensor(pipeline_name="the_pipeline", asset_key=AssetKey("foo"))
+def asset_foo_sensor(context, _event):
+    return RunRequest(run_key=context.cursor, run_config={})
+
+
+@asset_sensor(asset_key=AssetKey("foo"), job=the_job)
+def asset_job_sensor(context, _event):
+    return RunRequest(run_key=context.cursor, run_config={})
+
+
 @repository
 def the_repo():
     return [
         the_pipeline,
+        the_job,
+        config_pipeline,
+        foo_pipeline,
+        large_sensor,
         simple_sensor,
         error_sensor,
         wrong_config_sensor,
         always_on_sensor,
         run_key_sensor,
         custom_interval_sensor,
+        skip_cursor_sensor,
+        run_cursor_sensor,
+        asset_foo_sensor,
+        asset_job_sensor,
     ]
 
 
@@ -101,9 +206,10 @@ def the_other_repo():
 @contextmanager
 def instance_with_sensors(external_repo_context, overrides=None):
     with instance_for_test(overrides) as instance:
-        with ProcessGrpcServerRegistry(wait_for_processes_on_exit=True) as grpc_server_registry:
-            with external_repo_context() as external_repo:
-                yield (instance, grpc_server_registry, external_repo)
+        with ProcessGrpcServerRegistry() as grpc_server_registry:
+            with DynamicWorkspace(grpc_server_registry) as workspace:
+                with external_repo_context() as external_repo:
+                    yield (instance, workspace, external_repo)
 
 
 @contextmanager
@@ -117,24 +223,22 @@ def default_repo():
     with ManagedGrpcPythonEnvRepositoryLocationOrigin(
         loadable_target_origin=loadable_target_origin,
         location_name="test_location",
-    ).create_handle() as handle:
-        yield handle.create_location().get_repository("the_repo")
+    ).create_test_location() as location:
+        yield location.get_repository("the_repo")
 
 
 def repos():
     return [default_repo]
 
 
-def evaluate_sensors(instance, grpc_server_registry):
-
-    with RepositoryLocationHandleManager(grpc_server_registry) as handle_manager:
-        list(
-            execute_sensor_iteration(
-                instance,
-                get_default_daemon_logger("SensorDaemon"),
-                handle_manager,
-            )
+def evaluate_sensors(instance, workspace):
+    list(
+        execute_sensor_iteration(
+            instance,
+            get_default_daemon_logger("SensorDaemon"),
+            workspace,
         )
+    )
 
 
 def validate_tick(
@@ -151,7 +255,8 @@ def validate_tick(
     assert tick_data.job_type == JobType.SENSOR
     assert tick_data.status == expected_status
     assert tick_data.timestamp == expected_datetime.timestamp()
-    assert set(tick_data.run_ids) == set(expected_run_ids if expected_run_ids else [])
+    if expected_run_ids is not None:
+        assert set(tick_data.run_ids) == set(expected_run_ids)
     if expected_error:
         assert expected_error in tick_data.error.message
 
@@ -190,7 +295,7 @@ def test_simple_sensor(external_repo_context, capfd):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -202,7 +307,7 @@ def test_simple_sensor(external_repo_context, capfd):
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
@@ -225,7 +330,7 @@ def test_simple_sensor(external_repo_context, capfd):
             freeze_datetime = freeze_datetime.add(seconds=30)
 
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             wait_for_all_runs_to_start(instance)
             assert instance.get_runs_count() == 1
             run = instance.get_runs()[0]
@@ -264,7 +369,7 @@ def test_bad_load_sensor_repository(external_repo_context, capfd):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -287,7 +392,7 @@ def test_bad_load_sensor_repository(external_repo_context, capfd):
             ticks = instance.get_job_ticks(invalid_repo_origin.get_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(invalid_repo_origin.get_id())
@@ -309,7 +414,7 @@ def test_bad_load_sensor(external_repo_context, capfd):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -329,7 +434,7 @@ def test_bad_load_sensor(external_repo_context, capfd):
             ticks = instance.get_job_ticks(invalid_repo_origin.get_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(invalid_repo_origin.get_id())
@@ -348,7 +453,7 @@ def test_error_sensor(external_repo_context, capfd):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -360,7 +465,7 @@ def test_error_sensor(external_repo_context, capfd):
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
@@ -375,8 +480,6 @@ def test_error_sensor(external_repo_context, capfd):
             )
 
             captured = capfd.readouterr()
-            assert ("Failed to resolve sensor for error_sensor : ") in captured.out
-
             assert (
                 "Error occurred during the execution of evaluation_fn for sensor error_sensor"
             ) in captured.out
@@ -397,7 +500,7 @@ def test_wrong_config_sensor(external_repo_context, capfd):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -409,7 +512,7 @@ def test_wrong_config_sensor(external_repo_context, capfd):
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 1
@@ -428,7 +531,7 @@ def test_wrong_config_sensor(external_repo_context, capfd):
 
             # Error repeats on subsequent ticks
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 2
@@ -460,7 +563,7 @@ def test_launch_failure(external_repo_context, capfd):
                 "class": "ExplodingRunLauncher",
             },
         },
-    ) as (instance, grpc_server_registry, external_repo):
+    ) as (instance, workspace, external_repo):
         with pendulum.test(freeze_datetime):
 
             external_sensor = external_repo.get_external_sensor("always_on_sensor")
@@ -471,7 +574,7 @@ def test_launch_failure(external_repo_context, capfd):
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
 
             assert instance.get_runs_count() == 1
             run = instance.get_runs()[0]
@@ -505,7 +608,7 @@ def test_launch_once(external_repo_context, capfd):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -518,7 +621,7 @@ def test_launch_once(external_repo_context, capfd):
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             wait_for_all_runs_to_start(instance)
 
             assert instance.get_runs_count() == 1
@@ -536,7 +639,7 @@ def test_launch_once(external_repo_context, capfd):
         # run again (after 30 seconds), to ensure that the run key maintains idempotence
         freeze_datetime = freeze_datetime.add(seconds=30)
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             assert instance.get_runs_count() == 1
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 2
@@ -565,7 +668,7 @@ def test_launch_once(external_repo_context, capfd):
             # Sensor loop still executes
         freeze_datetime = freeze_datetime.add(seconds=30)
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
 
             assert len(ticks) == 3
@@ -584,7 +687,7 @@ def test_custom_interval_sensor(external_repo_context):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -595,7 +698,7 @@ def test_custom_interval_sensor(external_repo_context):
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 1
             validate_tick(ticks[0], external_sensor, freeze_datetime, JobTickStatus.SKIPPED)
@@ -603,7 +706,7 @@ def test_custom_interval_sensor(external_repo_context):
             freeze_datetime = freeze_datetime.add(seconds=30)
 
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             # no additional tick created after 30 seconds
             assert len(ticks) == 1
@@ -611,7 +714,7 @@ def test_custom_interval_sensor(external_repo_context):
             freeze_datetime = freeze_datetime.add(seconds=30)
 
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 2
 
@@ -635,7 +738,7 @@ def test_custom_interval_sensor_with_offset(external_repo_context, monkeypatch):
 
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -648,13 +751,13 @@ def test_custom_interval_sensor_with_offset(external_repo_context, monkeypatch):
             )
 
             # create a tick
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 1
 
             # calling for another iteration should not generate another tick because time has not
             # advanced
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
             assert len(ticks) == 1
 
@@ -663,7 +766,7 @@ def test_custom_interval_sensor_with_offset(external_repo_context, monkeypatch):
             list(
                 execute_sensor_iteration_loop(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     get_default_daemon_logger("SensorDaemon"),
                     until=freeze_datetime.add(seconds=65).timestamp(),
                 )
@@ -705,17 +808,24 @@ def test_error_sensor_daemon(external_repo_context, monkeypatch):
                 "class": "ExplodingRunLauncher",
             },
         },
-    ) as (instance, grpc_server_registry, _external_repo):
+    ) as (instance, workspace, _external_repo):
+
+        @contextmanager
+        def _gen_workspace(_instance):
+            yield workspace
+
         with pendulum.test(freeze_datetime):
             instance.add_job_state(
                 JobState(_get_unloadable_sensor_origin(), JobType.SENSOR, JobStatus.RUNNING)
             )
-            sensor_daemon = SensorDaemon.create_from_instance(instance)
+            sensor_daemon = SensorDaemon(interval_seconds=DEFAULT_SENSOR_DAEMON_INTERVAL)
             daemon_shutdown_event = threading.Event()
             sensor_daemon.run_loop(
                 "my_uuid",
                 daemon_shutdown_event,
-                grpc_server_registry,
+                _gen_workspace,
+                heartbeat_interval_seconds=DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
+                error_interval_seconds=DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
                 until=freeze_datetime.add(seconds=65),
             )
 
@@ -723,7 +833,7 @@ def test_error_sensor_daemon(external_repo_context, monkeypatch):
             heartbeat = heartbeats["SENSOR"]
             assert heartbeat
             assert heartbeat.errors
-            assert len(heartbeat.errors) == 1
+            assert len(heartbeat.errors) == DAEMON_HEARTBEAT_ERROR_LIMIT
 
 
 @pytest.mark.parametrize("external_repo_context", repos())
@@ -734,7 +844,7 @@ def test_sensor_start_stop(external_repo_context):
     )
     with instance_with_sensors(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -746,7 +856,7 @@ def test_sensor_start_stop(external_repo_context):
             ticks = instance.get_job_ticks(external_origin_id)
             assert len(ticks) == 0
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
 
             assert instance.get_runs_count() == 1
             run = instance.get_runs()[0]
@@ -759,7 +869,7 @@ def test_sensor_start_stop(external_repo_context):
             freeze_datetime = freeze_datetime.add(seconds=15)
 
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             # no new ticks, no new runs, we are below the 30 second min interval
             assert instance.get_runs_count() == 1
             ticks = instance.get_job_ticks(external_origin_id)
@@ -769,7 +879,7 @@ def test_sensor_start_stop(external_repo_context):
             instance.stop_sensor(external_origin_id)
             instance.start_sensor(external_sensor)
 
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             # no new ticks, no new runs, we are below the 30 second min interval
             assert instance.get_runs_count() == 1
             ticks = instance.get_job_ticks(external_origin_id)
@@ -778,8 +888,192 @@ def test_sensor_start_stop(external_repo_context):
             freeze_datetime = freeze_datetime.add(seconds=16)
 
         with pendulum.test(freeze_datetime):
-            evaluate_sensors(instance, grpc_server_registry)
+            evaluate_sensors(instance, workspace)
             # should have new tick, new run, we are after the 30 second min interval
             assert instance.get_runs_count() == 2
             ticks = instance.get_job_ticks(external_origin_id)
             assert len(ticks) == 2
+
+
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_large_sensor(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_sensors(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+            external_sensor = external_repo.get_external_sensor("large_sensor")
+            instance.start_sensor(external_sensor)
+            evaluate_sensors(instance, workspace)
+            ticks = instance.get_job_ticks(external_sensor.get_external_origin_id())
+            assert len(ticks) == 1
+            validate_tick(
+                ticks[0],
+                external_sensor,
+                freeze_datetime,
+                JobTickStatus.SUCCESS,
+            )
+
+
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_cursor_sensor(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_sensors(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+            skip_sensor = external_repo.get_external_sensor("skip_cursor_sensor")
+            run_sensor = external_repo.get_external_sensor("run_cursor_sensor")
+            instance.start_sensor(skip_sensor)
+            instance.start_sensor(run_sensor)
+            evaluate_sensors(instance, workspace)
+
+            skip_ticks = instance.get_job_ticks(skip_sensor.get_external_origin_id())
+            assert len(skip_ticks) == 1
+            validate_tick(
+                skip_ticks[0],
+                skip_sensor,
+                freeze_datetime,
+                JobTickStatus.SKIPPED,
+            )
+            assert skip_ticks[0].cursor == "1"
+
+            run_ticks = instance.get_job_ticks(run_sensor.get_external_origin_id())
+            assert len(run_ticks) == 1
+            validate_tick(
+                run_ticks[0],
+                run_sensor,
+                freeze_datetime,
+                JobTickStatus.SUCCESS,
+            )
+            assert run_ticks[0].cursor == "1"
+
+        freeze_datetime = freeze_datetime.add(seconds=60)
+        with pendulum.test(freeze_datetime):
+            evaluate_sensors(instance, workspace)
+
+            skip_ticks = instance.get_job_ticks(skip_sensor.get_external_origin_id())
+            assert len(skip_ticks) == 2
+            validate_tick(
+                skip_ticks[0],
+                skip_sensor,
+                freeze_datetime,
+                JobTickStatus.SKIPPED,
+            )
+            assert skip_ticks[0].cursor == "2"
+
+            run_ticks = instance.get_job_ticks(run_sensor.get_external_origin_id())
+            assert len(run_ticks) == 2
+            validate_tick(
+                run_ticks[0],
+                run_sensor,
+                freeze_datetime,
+                JobTickStatus.SUCCESS,
+            )
+            assert run_ticks[0].cursor == "2"
+
+
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_asset_sensor(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_sensors(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+            foo_sensor = external_repo.get_external_sensor("asset_foo_sensor")
+            instance.start_sensor(foo_sensor)
+
+            evaluate_sensors(instance, workspace)
+
+            ticks = instance.get_job_ticks(foo_sensor.get_external_origin_id())
+            assert len(ticks) == 1
+            validate_tick(
+                ticks[0],
+                foo_sensor,
+                freeze_datetime,
+                JobTickStatus.SKIPPED,
+            )
+
+            freeze_datetime = freeze_datetime.add(seconds=60)
+        with pendulum.test(freeze_datetime):
+
+            # should generate the foo asset
+            execute_pipeline(foo_pipeline, instance=instance)
+
+            # should fire the asset sensor
+            evaluate_sensors(instance, workspace)
+            ticks = instance.get_job_ticks(foo_sensor.get_external_origin_id())
+            assert len(ticks) == 2
+            validate_tick(
+                ticks[0],
+                foo_sensor,
+                freeze_datetime,
+                JobTickStatus.SUCCESS,
+            )
+            run = instance.get_runs()[0]
+            assert run.run_config == {}
+            assert run.tags
+            assert run.tags.get("dagster/sensor_name") == "asset_foo_sensor"
+
+
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_asset_job_sensor(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_sensors(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+            job_sensor = external_repo.get_external_sensor("asset_job_sensor")
+            instance.start_sensor(job_sensor)
+
+            evaluate_sensors(instance, workspace)
+
+            ticks = instance.get_job_ticks(job_sensor.get_external_origin_id())
+            assert len(ticks) == 1
+            validate_tick(
+                ticks[0],
+                job_sensor,
+                freeze_datetime,
+                JobTickStatus.SKIPPED,
+            )
+
+            freeze_datetime = freeze_datetime.add(seconds=60)
+        with pendulum.test(freeze_datetime):
+
+            # should generate the foo asset
+            execute_pipeline(foo_pipeline, instance=instance)
+
+            # should fire the asset sensor
+            evaluate_sensors(instance, workspace)
+            ticks = instance.get_job_ticks(job_sensor.get_external_origin_id())
+            assert len(ticks) == 2
+            validate_tick(
+                ticks[0],
+                job_sensor,
+                freeze_datetime,
+                JobTickStatus.SUCCESS,
+            )
+            run = instance.get_runs()[0]
+            assert run.run_config == {}
+            assert run.tags
+            assert run.tags.get("dagster/sensor_name") == "asset_job_sensor"
